/******************************************************************************\
*                                  JZip                                        *
*                                                                              *
* COPYRIGHT:                                                                   *
*   (C) Copyright 1999-2000 Cramon Utilities and Bytamin-C                     *
*                                                                              *
* WWW:                                                                         *
*   http://www.bytamin-c.com                                                   *
*                                                                              *
* DISCLAMER:                                                                   *
*   THE AUTHOR(S) MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY *
*   OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO   *
*   THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR        *
*   PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR(S) SHALL NOT BE LIABLE FOR ANY    *
*   DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR            *
*   DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.                             *
*                                                                              *
* ---------------------------------------------------------------------------- *
* Last changed      Name                   Changes                             *
* 01/13-2000        Jeppe Cramon           Prepared for OpenSource release     *
\******************************************************************************/

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <io.h>
#include <fcntl.h>
#include "filectrl.hpp"
#include "MainUnit.h"
#include "AddUnit.h"
#include "ExtractUnit.h"
#include "InstallUnit.h"
#include "CommentUnit.h"
#include "PasswordUnit.h"
#include "ExtractPasswUnit.h"
#include "TestUnit.h"
#include "AboutUnit.h"
#include "EXEUnit.h"
#include "ConfigurationUnit.h"
#include "DragDropUnit.h"
#include "OverwriteUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "ZipBuilder"
#pragma link "Placemnt"
#pragma link "ExtListView"
#pragma link "SystemImageList"
#pragma link "DropSource"
#pragma link "DropTarget"
#pragma link "dfsStatusBar"
#pragma resource "*.dfm"
TMainForm *MainForm;
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
    // Initialize the random engine
    Randomize();

    // Init values
    TotalBytes = 0;
    TotalSelected = 0;
    bExtractSelected = false;
    bYesToAll = false;
    bTesting = false;
    iTestErrors = 0;
    bInProgress = false;
    bDragging = false;
    bDraggingOverUs = false;

    // Allocate a new instance of TStringList
    CleanUpList = new TStringList();
}
//---------------------------------------------------------------------------
__fastcall TMainForm::~TMainForm()
{
    // Clean up tmp dirs
    if (CleanUpList->Count)
    {
        for (int iLoop=0; iLoop < CleanUpList->Count; iLoop++)
        {
            DeleteDir(CleanUpList->Strings[iLoop]);
        }
    }
    CleanUpList->Clear();
    delete CleanUpList;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::OpenArchive1Click(TObject *Sender)
{
    // Show Open dialog
    if (OpenDialog->Execute())
    {
        Caption = "JZip - "+MinimizeName( OpenDialog->FileName, Canvas, ClientWidth-Canvas->TextWidth("JZip - ")-140);
        ZipBuilder->ZipFileName = OpenDialog->FileName;
        ZipBuilder->Password = "";
        Update();
    }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::OpenToolButtonClick(TObject *Sender)
{
    // Perform Open
    OpenArchive1Click(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Update()
{
    // Init values
    TotalBytes = 0;
    TotalSelected = 0;
    InstallToolButton->Enabled = false;
    InstallProgram = "";

    // Clear the List
    ExtListView->Items->Clear();

    // Fill the Grid with the contents of the ZipFile (if any file exists)
    if (ZipBuilder->ZipFileName == "")
    {
        AddToolButton->Enabled = false;
        ExtractToolButton->Enabled = false;
        ViewToolButton->Enabled = false;
//        CheckOutToolButton->Enabled = false; (kept for later use)
        InstallToolButton->Enabled = false;
        StatusBar->SimplePanel = true;

        // Menu bar
        Delete1->Enabled = false;
        CloseArchive1->Enabled = false;
        CreateShortcut1->Enabled = false;
        MoveArchive1->Enabled = false;
        CopyArchive1->Enabled = false;
        RenameArchive1->Enabled = false;
        DeleteArchive1->Enabled = false;
        //-----
        Add1->Enabled = false;
        Extract1->Enabled = false;
        View1->Enabled = false;
        SelectAll1->Enabled = false;
        InvertSelection1->Enabled = false;
        MakeEXEFile1->Enabled = false;
        Test1->Enabled = false;
        Comment1->Enabled = false;
        Install1->Enabled = false;
        //-----
        Password1->Enabled = false;

        // Popup menu
        OpenFile1->Enabled = false;
        Delete2->Enabled = false;
        Add2->Enabled = false;
        Extract2->Enabled = false;
        View2->Enabled = false;
        SelectAll2->Enabled = false;
        InvertSelection2->Enabled = false;
        MakeEXEFile2->Enabled = false;
        Test2->Enabled = false;
        Comment2->Enabled = false;
        Install2->Enabled = false;

        return;
    }
    else
    {
        // Menu bar
        Delete1->Enabled = false;
        CloseArchive1->Enabled = true;
        // ----
        Add1->Enabled = true;
        Extract1->Enabled = true;
        View1->Enabled = false;
        SelectAll1->Enabled = true;
        InvertSelection1->Enabled = true;
        MakeEXEFile1->Enabled = true;
        Test1->Enabled = true;
        Comment1->Enabled = true;
        Install1->Enabled = false;
        Password1->Enabled = true;

        // Popup menu
        OpenFile1->Enabled = true;
        Delete2->Enabled = true;
        Add2->Enabled = true;
        Extract2->Enabled = true;
        View2->Enabled = true;
        SelectAll2->Enabled = true;
        InvertSelection2->Enabled = true;
        MakeEXEFile2->Enabled = true;
        Test2->Enabled = true;
        Comment2->Enabled = true;
        Install2->Enabled = false;

        // Toolbar
        AddToolButton->Enabled = true;
        ViewToolButton->Enabled = false;
        InstallToolButton->Enabled = false;

        // Statusbar
        StatusBar->SimplePanel = false;

        if (ZipBuilder->Count > 0)
        {
            ExtractToolButton->Enabled = true;
//            CheckOutToolButton->Enabled = true; (kept for later use)
        }
        else
        {
            ExtractToolButton->Enabled = false;
//            CheckOutToolButton->Enabled = false; (kept for later use)
        }

    }

    // ---- Fill ListView ---- //
    ZipDirEntry *zde;
    AnsiString FileName;
    TListItem *Item;
    for (int iLoop=0; iLoop < ZipBuilder->Count; iLoop++)
    {
        zde = (ZipDirEntry *)ZipBuilder->ZipContents->Items[ iLoop ];

        // "Names"
        FileName = ExtractFileName(zde->FileName);
        Item = ExtListView->Items->Add();
        Item->Caption = FileName;

        // Set icon
        Item->ImageIndex = SysImgList->GetImageIndex(FileName, false, false, TSystemFileAttributes());

        // "Modified"
        try
        {
            Item->SubItems->Add(FormatDateTime( "ddddd  t", FileDateToDateTime( zde->DateTime ) ));
        }
        catch(...)
        {
            Item->SubItems->Add("Error...");
        }

        // "Size"
        Item->SubItems->Add(IntToStr((int)zde->UncompressedSize));
        // "Ratio"
        if (zde->UncompressedSize)
            Item->SubItems->Add(AnsiString(FloatToStr(100.0-((float)zde->CompressedSize*100.0/(float)zde->UncompressedSize))).SubString(0,5)+"%");
        else
            Item->SubItems->Add("0%");
        // "Packed"
        Item->SubItems->Add( IntToStr((int)zde->CompressedSize) );
        // "Path"
        Item->SubItems->Add( ExtractFileDir(zde->FileName) );

        // Update Total Size
        TotalBytes += zde->UncompressedSize;

        // Check if we should update the install button
        if (FileName.LowerCase() == "setup.exe" || FileName.LowerCase() == "install.exe")
        {
            InstallToolButton->Enabled = true;
            Install1->Enabled = true;
            Install2->Enabled = true;
            InstallProgram = FileName;
        }

    } // end for

    // Update status bar
    if (TotalBytes/1024 > 1)
        StatusBar->Panels->Items[1]->Text = "Total "+IntToStr(ZipBuilder->Count)+" files, "+IntToStr((int)TotalBytes/1024)+" KB";
    else
        StatusBar->Panels->Items[1]->Text = "Total "+IntToStr(ZipBuilder->Count)+" files, "+IntToStr((int)TotalBytes)+" bytes";

}
//---------------------------------------------------------------------------
void __fastcall TMainForm::NewArchive1Click(TObject *Sender)
{
    // Show new dialog
    if (NewDialog->Execute())
    {
        Caption = "JZip - "+MinimizeName( NewDialog->FileName, Canvas, ClientWidth-Canvas->TextWidth("JZip - ")-140);
        ZipBuilder->ZipFileName = NewDialog->FileName;
        ZipBuilder->Password = "";
        Update();
    }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::NewToolButtonClick(TObject *Sender)
{
    // Perform New
    NewArchive1Click(NULL);
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::Add1Click(TObject *Sender)
{
    // Init AddForm vars and components
    AddForm->AddBitBtn->Enabled = false;
    AddForm->SystemListView->Selected = NULL;
    AddForm->NameEdit->Text = AddForm->DefaultWildCard;
    AddForm->Password = "";

    // Show it
    AddForm->ShowModal();
    if (AddForm->ModalResult == mrCancel)
        return;

    /* --- Steps that are default for both with and without wildcard --- */
    // Set compression level
    switch(AddForm->CompressionComboBox->ItemIndex)
    {
        case 0 : // Maximum
            ZipBuilder->AddCompLevel = 9;
            break;
        case 1 : // Normal
            ZipBuilder->AddCompLevel = 6;
            break;
        case 2 : // Fast
            ZipBuilder->AddCompLevel = 3;
            break;
        case 3 : // Superfast
            ZipBuilder->AddCompLevel = 2;
            break;
        case 4 : // None
            ZipBuilder->AddCompLevel = 0;
            break;
    } // end switch

    // Set Action type
    ZipBuilder->AddOptions.Clear();
    switch(AddForm->ActionComboBox->ItemIndex)
    {
        case 0: // Add (and replace)
            // Don nothing. This is the default
            break;
        case 1: // Freshen existing files
            ZipBuilder->AddOptions << AddFreshen;
            break;
        case 2: // Move files
            ZipBuilder->AddOptions << AddMove;
            break;
        case 3: // Update and add
            ZipBuilder->AddOptions << AddUpdate;
            break;
    } // end switch

    // Store 8.3 names
    if (AddForm->ForceDOSCheckBox->Checked)
        ZipBuilder->AddOptions << AddForceDOS;

    // Include SubFolders
    if (AddForm->SubFldrCheckBox->Checked)
    {
        ZipBuilder->AddOptions << AddRecurseDirs;
        ZipBuilder->AddOptions << AddDirNames;
    }

    // Save extra folder info
    if (AddForm->ExtraFldrCheckBox->Checked)
        ZipBuilder->AddOptions << AddDirNames;

    // Include System and hidden files
    if (AddForm->SysFilesCheckBox->Checked)
        ZipBuilder->AddOptions << AddHiddenFiles;

    // Password
    if (AddForm->Password != "")
        ZipBuilder->Password = AddForm->Password;
    if (ZipBuilder->Password != "")
        ZipBuilder->AddOptions << AddEncrypt;

    // Check for special request by user
    if (AddForm->SubFldrCheckBox->Checked && !AddForm->ExtraFldrCheckBox->Checked)
    {
        SetCurrentDir(AddForm->RootDir);
        AddForm->RootDir = "";
    }

    ZipBuilder->FSpecArgs->Clear();
    // Check if it's Add with WildCard
    if (AddForm->ModalResult == mrYes)
    {
        if (AddForm->RootDir=="")
            ZipBuilder->FSpecArgs->Add(AddForm->NameEdit->Text);
        else
            ZipBuilder->FSpecArgs->Add(AddForm->RootDir+AddForm->NameEdit->Text);
    } // else it's just plain Add
    else if (AddForm->ModalResult == mrOk)
    {
        ZipBuilder->FSpecArgs->CommaText = AddForm->NameEdit->Text;

        for (int iLoop=0; iLoop < ZipBuilder->FSpecArgs->Count; iLoop++)
        {
            ZipBuilder->FSpecArgs->Strings[iLoop] = AddForm->RootDir+ZipBuilder->FSpecArgs->Strings[iLoop];

            // Check if it's a directory we're adding
            if (AddForm->IsDirectory(ZipBuilder->FSpecArgs->Strings[iLoop]))
            {
                ZipBuilder->FSpecArgs->Strings[iLoop] = ZipBuilder->FSpecArgs->Strings[iLoop]+"\\*.*";
            }
        }
    }

    // --- Do the works ;) --- //
    ZipBuilder->Add();

    // Update the system
    Update();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::AddToolButtonClick(TObject *Sender)
{
    // Perform Add
    Add1Click(NULL);
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ExtListViewChange(TObject *Sender,
      TListItem *Item, TItemChange Change)
{
    // Init Values
    TListItem *SelItem = ExtListView->Selected;
    TItemStates SearchState;
    SearchState << isSelected;

    // Set menues and toolbar status' according to selection
    if (ExtListView->SelCount==1)
    {
        ViewToolButton->Enabled = true;
        View1->Enabled = true;
        Delete1->Enabled = true;

        OpenFile1->Enabled = true;
        View2->Enabled = true;
        Delete2->Enabled = true;

    }
    else
    {
        ViewToolButton->Enabled = false;
        OpenFile1->Enabled = false;
        View1->Enabled = false;
        View2->Enabled = false;
        if (ExtListView->SelCount)
        {
            Delete1->Enabled = true;
            Delete2->Enabled = true;
        }
        else
        {
            Delete1->Enabled = false;
            Delete2->Enabled = false;
        }
    }

    // Count selected bytes
    TotalSelected = 0;
    while( SelItem )
    {
        TotalSelected += StrToInt( SelItem->SubItems->Strings[1]);
        SelItem = ExtListView->GetNextItem(SelItem, sdAll, SearchState);
    }

    // Update StatusBar
    if (TotalSelected/1024 > 1)
        StatusBar->Panels->Items[0]->Text = "Total "+IntToStr(ExtListView->SelCount)+" files, "+IntToStr((int)TotalSelected/1024)+" KB";
    else
        StatusBar->Panels->Items[0]->Text = "Total "+IntToStr(ExtListView->SelCount)+" files, "+IntToStr((int)TotalSelected)+" bytes";

}
//---------------------------------------------------------------------------

void __fastcall TMainForm::SelectAll1Click(TObject *Sender)
{
    // Select all items
    TListItem *Item;
    for (int iLoop=0; iLoop < ExtListView->Items->Count; iLoop++)
    {
        Item = ExtListView->Items->Item[iLoop];
        Item->Selected = true;
    }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::InvertSelection1Click(TObject *Sender)
{
    // Invert selection
    TListItem *Item;
    for (int iLoop=0; iLoop < ExtListView->Items->Count; iLoop++)
    {
        Item = ExtListView->Items->Item[iLoop];
        Item->Selected = !Item->Selected;
    }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FormCreate(TObject *Sender)
{
    // Check if the user has answered yes to the license agreement
    TRegistry* thisReg = new TRegistry();
    thisReg->OpenKey("SOFTWARE\\JZip", true);
    if (thisReg->ReadString("") != "1")
    {
        if (Application->MessageBox( "This software is provided \"as is\" without warranty of any kind, "
                                    "either expressed or implied.  the entire risk as to the "
                                    "quality and performance of the software is with you. Should the "
                                    "software prove defective, you assume the cost of all necessary "
                                    "servicing, repair, or correction.  in no event shall the author, "
                                    "copyright holder, or any other party who may redistribute the "
                                    "software be liable to you for damages, including any general, "
                                    "special, incidental, or consequential damages arising out of "
                                    "the use or inability to use the software (including, but not "
                                    "limited to, loss of data, data being rendered inaccurate, loss of "
                                    "business profits, loss of business information, business "
                                    "interruptions, loss sustained by you or third parties, or a "
                                    "failure of the software to operate with any other software) even "
                                    "if the author, copyright holder, or other party has been advised "
                                    "of the possibility of such damages.\n\nJZip is released under GPL license."
                                    "Press \"Yes\" if you accept this disclaimer agreement!",
                                     "JZip disclaimer", MB_YESNO | MB_ICONWARNING) == ID_YES)
        {
            thisReg->WriteString("", "1");
            thisReg->CloseKey();
        }
        else
        {
            thisReg->WriteString("", "");
            thisReg->CloseKey();
            delete thisReg;
            Application->Terminate();
            return;
        }
    }
    else
    {
        thisReg->CloseKey();
    }
    // Clean up
    delete thisReg;

    // Set zip DLL dir
    if (FileExists(ExtractFilePath(ParamStr(0))+"UnzDll.dll"))
    {
        ZipBuilder->DLLDirectory = ExtractFilePath(ParamStr(0));
    }

    /* --- Read configuration --- */
    ReadConfiguration();
    SetConfiguration();

    // Check if the user needs a newer version of COMCTL32.DLL
    if (ExtListView->RequireComCtlUpdate)
    {
        Application->MessageBox("You need a newer version of COMCTL32.DLL to run JZip properly!", "Version Check", MB_OK | MB_ICONWARNING);
    }

    // Set drag/drop support
    DropFileTarget->Register(ExtListView);


    // Get temp dir
    LPTSTR szTmpDir = new TCHAR[MAX_PATH];
    GetTempPath(MAX_PATH, szTmpDir);
    ZipBuilder->TempDir = szTmpDir;
    delete szTmpDir;

    // DLL's
    ZipBuilder->DLLDirectory = ExtractFilePath(ParamStr(0));
    ZipBuilder->Load_Zip_Dll();
    ZipBuilder->Load_Unz_Dll();

    // Get startup directory
    StartupDirectory = GetCurrentDir();
    // Set SFX dir
    ZipBuilder->SFXPath = ExtractFilePath(ParamStr(0));
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
    // Unregister drag/drop
    DropFileTarget->Unregister();

    // Unload ZIP dlls
    ZipBuilder->Unload_Zip_Dll();
    ZipBuilder->Unload_Unz_Dll();

    // Save configuration
    WriteConfiguration();
}
//---------------------------------------------------------------------------
// Update Progress bar
void __fastcall TMainForm::ZipBuilderProgress(TObject *Sender,
      ProgressType ProgrType, AnsiString FileName, long FileSize)
{
    // Local statis vars (made static for speed)
    static long TotalSize;
    static long BytesProcessed;
    static float fTmp;

    switch(ProgrType)
    {
        case TotalSize2Process :

            // Start
            bInProgress = true;
            bYesToAll = false;

            // Set everything to Cursor Hourglass
            Cursor = crHourGlass;
            ToolBar->Cursor = crHourGlass;
            ExtListView->Cursor = crHourGlass;
            StatusBar->Cursor = crHourGlass;
            // Disable components
            File1->Enabled = false;
            Actions1->Enabled = false;
            Options1->Enabled = false;
            Help1->Enabled = false;
            ToolBar->Enabled = false;
            NewToolButton->Enabled = false;
            OpenToolButton->Enabled = false;
            AddToolButton->Enabled = false;
            ExtractToolButton->Enabled = false;
            ViewToolButton->Enabled = false;
            InstallToolButton->Enabled = false;
            AboutToolButton->Enabled = false;
            ExtListView->Enabled = false;

            // Set 2. panel to Gauge
            StatusBar->Panels->Items[1]->PanelType = sptGauge;
            StatusBar->Panels->Items[1]->GaugeAttrs->Position = 0;
            StatusBar->Panels->Items[1]->Text = "";

            // Update TotalSize
            // Workaround for bug in TZipBuilder
            if (bExtractSelected)
                TotalSize = TotalSelected;
            else
                TotalSize = FileSize;
            BytesProcessed=0;
            break;
        case NewFile:
            // Set text in panel 1
            StatusBar->Panels->Items[0]->Text = "Processing: "+FileName;
            break;
        case ProgressUpdate :
            // Update Bytes processed
            BytesProcessed += FileSize;

            if (TotalSize > 0)
            {
                // Update Gauge
                fTmp = (float)BytesProcessed/TotalSize;
                fTmp *= 100.0;
                StatusBar->Panels->Items[1]->GaugeAttrs->Position = (int)fTmp;
            }

            // Make the app live a little to let the user be able to cancel the operation
            Application->ProcessMessages();
            break;
        case EndOfBatch :
            // End
            bInProgress = false;
            bYesToAll = false;

            // Set 2. panel back to Text
            StatusBar->Panels->Items[1]->PanelType = sptNormal;
            StatusBar->Panels->Items[1]->Alignment = taLeftJustify;
            bExtractSelected = false;   // Bug workaround

            // Restore Current directory
            if (AddForm->SubFldrCheckBox->Checked && !AddForm->ExtraFldrCheckBox->Checked)
                SetCurrentDir(StartupDirectory);

            // Set everything back to normal cursors
            Cursor = crDefault;
            ToolBar->Cursor = crDefault;
            ExtListView->Cursor = crDefault;
            StatusBar->Cursor = crDefault;

            File1->Enabled = true;
            Actions1->Enabled = true;
            Options1->Enabled = true;
            Help1->Enabled = true;
            ToolBar->Enabled = true;
            NewToolButton->Enabled = true;
            OpenToolButton->Enabled = true;
            AddToolButton->Enabled = true;
            ExtractToolButton->Enabled = true;
            AboutToolButton->Enabled = true;
            ExtListView->Enabled = true;

            break;
    } // end switch
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Extract1Click(TObject *Sender)
{
    // Set buttons
    if (ExtListView->Selected == NULL)
    {
        ExtractForm->SelRadioButton->Enabled = false;
        ExtractForm->AllRadioButton->Checked = true;
    }
    else
    {
        ExtractForm->SelRadioButton->Enabled = true;
        ExtractForm->SelRadioButton->Checked = true;
    }

    // Show
    ExtractForm->ShowModal();
    if (ExtractForm->ModalResult == mrCancel)
        return;

    // Set options
    ZipBuilder->ExtrOptions.Clear();
    if (ExtractForm->UserFolderNamesCheckBox->Checked)
        ZipBuilder->ExtrOptions << ExtrDirNames;

    if (ExtractForm->SkipOlderCheckBox->Checked)
        ZipBuilder->ExtrOptions << ExtrUpdate;

    if (ExtractForm->OverwriteCheckBox->Checked)
        ZipBuilder->ExtrOptions << ExtrOverWrite;

    // Set FSpecArgs
    ZipBuilder->FSpecArgs->Clear();
    AnsiString asTmp;
    if (ExtractForm->SelRadioButton->Checked)
    {
        TListItem *SelItem = ExtListView->Selected;
        TItemStates SearchState;
        SearchState << isSelected;

        while( SelItem )
        {
            if (SelItem->SubItems->Strings[4] == "")
                asTmp = SelItem->Caption;
            else
                asTmp = SelItem->SubItems->Strings[4]+"\\"+SelItem->Caption;

            // Add it
            ZipBuilder->FSpecArgs->Add(asTmp);

            SelItem = ExtListView->GetNextItem(SelItem, sdAll, SearchState);
        }

        bExtractSelected = true;
    }
    else if (ExtractForm->AllRadioButton->Checked)
    {
        ZipBuilder->FSpecArgs->Add("*.*");
    }
    else // Files...
    {
        if (ExtractForm->FilesEdit->Text == "*")
            ZipBuilder->FSpecArgs->Add("*.*");
        else
            ZipBuilder->FSpecArgs->Add(ExtractForm->FilesEdit->Text);
    }

    // Make dir if it doesn't already exists
    ZipBuilder->ExtrBaseDir = ExtractForm->ExtractToComboBox->Text;
    ForceDirectories(ZipBuilder->ExtrBaseDir);
    if (!DirectoryExists(ZipBuilder->ExtrBaseDir))
    {
        Application->MessageBox("Error creating extraction directory!\nExtraction failed.", "JZip Error", MB_ICONERROR | MB_OK);
        return;
    }

    // Extract
    ZipBuilder->Extract();

    // Update the system
    Update();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ExtractToolButtonClick(TObject *Sender)
{
    // Perform Extract
    Extract1Click(NULL);
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ExtListViewMouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
    // Save Mouse coord (used in ExtListViewDblClick)
    iMouseX = X;
    iMouseY = Y;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ExtListViewDblClick(TObject *Sender)
{
    // Get the item that was double clicked
    TListItem* Item = ExtListView->GetItemAt(iMouseX, iMouseY);
    if (Item)
    {

        // Extract the file (unless if it's a setup.exe or install.exe file, then we extract the entire zip file)
        if (Item->Caption.LowerCase() == "setup.exe" || Item->Caption.LowerCase() == "install.exe")
        {
            // This is the same as Install
            Install1Click(NULL);

        }
        else
        {
            // Extract the file to the temp dir
            // Make temp dir
            AnsiString TempDir = CreateTempDir();
            // Add the temp dir to the CleanUpList
            CleanUpList->Add(TempDir);

            // Set options
            ZipBuilder->ExtrOptions.Clear();
            ZipBuilder->ExtrOptions << ExtrOverWrite;

            // Set FSpecArgs
            ZipBuilder->FSpecArgs->Clear();
            if (Item->SubItems->Strings[4] == "")
                ZipBuilder->FSpecArgs->Add(Item->Caption);
            else
                ZipBuilder->FSpecArgs->Add(Item->SubItems->Strings[4]+"\\"+Item->Caption);

            // Make dir if it doesn't already exists
            ZipBuilder->ExtrBaseDir = TempDir;
            CreateDir(ZipBuilder->ExtrBaseDir);
            // Save file to open before extraction and update
            AnsiString tmp = TempDir+"\\"+Item->Caption;

            ZipBuilder->Extract();
            Update();

            int rc = (int)   ShellExecute(Handle,
                                 "open",
                                 tmp.c_str(),
                                 NULL,NULL,SW_SHOWDEFAULT);
            if (rc <= 32)
            {
                if (rc == SE_ERR_NOASSOC)
                    Application->MessageBox("Sorry, there's no default viewer for this file type!","View Error", MB_OK | MB_ICONERROR);
                else
                    Application->MessageBox("An error ocurred while spawning the viewer application!","View Error", MB_OK | MB_ICONERROR);
            }
        }
    }
}
//---------------------------------------------------------------------------
// Create a temporary directory in the windows temp dir.
AnsiString __fastcall TMainForm::CreateTempDir()
{
    // Local vars
    AnsiString asTmp;
    char szTmp[MAX_PATH];
    int tmp;

    // Create a unique dir name (combined of numbers)
    // As an improvement letters could later be used, but it's not that crucial
    while(1)
    {
        asTmp = "";
        for(int iLoop=0;iLoop < 8; iLoop++)
        {
            tmp = random(58);
            while(tmp <48 || tmp > 57)
                tmp = random(58);
            sprintf(szTmp, "%c",tmp);
            asTmp += szTmp;
        }

        asTmp = ZipBuilder->TempDir+asTmp;
        if (!SetCurrentDir(asTmp))
        {
            CreateDir(asTmp);
            break;
        }
    } // end while

    // Return the dir name
    return asTmp;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::Install1Click(TObject *Sender)
{
    // Install
    // Set options
    ZipBuilder->ExtrOptions.Clear();
    ZipBuilder->ExtrOptions << ExtrDirNames;
    ZipBuilder->ExtrOptions << ExtrOverWrite;

    // Set FSpecArgs
    ZipBuilder->FSpecArgs->Clear();
    ZipBuilder->FSpecArgs->Add("*.*");

    // Make temp dir
    AnsiString TempDir = CreateTempDir();
    // Add the temp dir to the CleanUpList
    CleanUpList->Add(TempDir);

    // Make dir if it doesn't already exists
    ZipBuilder->ExtrBaseDir = TempDir;
    CreateDir(ZipBuilder->ExtrBaseDir);

    // Perform the extraction
    ZipBuilder->Extract();

    // Update the system
    Update();

    // Change dir to the extraction dir
    SetCurrentDir(TempDir);

    // Execute the installation program
    ShellExecute(Handle,
                 "open",
                 InstallProgram.c_str(),
                 NULL,NULL,SW_SHOWDEFAULT);

    // Show the install form
    InstallForm->ShowModal();

    // Delete files
    DeleteDir(TempDir);
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::InstallToolButtonClick(TObject *Sender)
{
    // Perform install
    Install1Click(NULL);
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::View1Click(TObject *Sender)
{
    // View a file
    // Get the item that was double clicked
    TListItem* Item = ExtListView->Selected;
    if (Item)
    {

        // Extract the file (unless if it's a setup.exe or install.exe file, then we extract the entire zip file)
        if (Item->Caption.LowerCase() == "setup.exe" || Item->Caption.LowerCase() == "install.exe")
        {
            // This is the same as Install
            Install1Click(NULL);

        }
        else
        {
            // Extract the file to the temp dir
            // Make temp dir
            AnsiString TempDir = CreateTempDir();
            // Add the temp dir to the CleanUpList
            CleanUpList->Add(TempDir);

            // Set options
            ZipBuilder->ExtrOptions.Clear();
            ZipBuilder->ExtrOptions << ExtrOverWrite;

            // Set FSpecArgs
            ZipBuilder->FSpecArgs->Clear();
            if (Item->SubItems->Strings[4] == "")
                ZipBuilder->FSpecArgs->Add(Item->Caption);
            else
                ZipBuilder->FSpecArgs->Add(Item->SubItems->Strings[4]+"\\"+Item->Caption);

            // Make dir if it doesn't already exists
            ZipBuilder->ExtrBaseDir = TempDir;
            CreateDir(ZipBuilder->ExtrBaseDir);
            // Save file to open before extraction and update
            AnsiString tmp = TempDir+"\\"+Item->Caption;

            // Set workaround flag
            bExtractSelected = true;

            // Perform extraction
            ZipBuilder->Extract();

            // Update system
            Update();


            int rc = (int)   ShellExecute(Handle,
                                 "open",
                                 tmp.c_str(),
                                 NULL,NULL,SW_SHOWDEFAULT);
            if (rc <= 32)
            {
                if (rc == SE_ERR_NOASSOC)
                    Application->MessageBox("Sorry, there's no default viewer for this file type!","View Error", MB_OK | MB_ICONERROR);
                else
                    Application->MessageBox("An error ocurred while spawning the viewer application!","View Error", MB_OK | MB_ICONERROR);
            }
        }
    }

}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ViewToolButtonClick(TObject *Sender)
{
    // Perform view
    View1Click(NULL);
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::CloseArchive1Click(TObject *Sender)
{
    // Close archive (reinit)
    ZipBuilder->ZipFileName = "";
    ZipBuilder->Password = "";
    Caption = "JZip";
    Update();
}
//---------------------------------------------------------------------------
// Delete a dir (and all files in it)
void __fastcall TMainForm::DeleteDir(AnsiString Dir)
{
    // Local static vars
    static char szTmp[MAX_PATH];

    AnsiString asTmpDir = GetCurrentDir();
    if (!SetCurrentDir(Dir))
        return;     // Error the directory didn't exist
    else
        SetCurrentDir(asTmpDir);

    StatusBar->Panels->Items[1]->GaugeAttrs->Style = gsIndeterminate2;
    StatusBar->Panels->Items[1]->PanelType = sptGauge;
    StatusBar->Panels->Items[1]->Enabled = true;


    // Set filename to be deleted (memset's used because SHFileOp expects double \0\0 as termination of string
    memset(szTmp, 0, MAX_PATH*sizeof(char));
    strcpy(szTmp, Dir.c_str());

    // Set up the struct
    SHFILEOPSTRUCT op;
    memset(&op,0,sizeof(SHFILEOPSTRUCT));
    op.hwnd  = Handle;  // Handle of main form or the application
    op.wFunc = FO_DELETE;
    op.pFrom = szTmp;
    op.pTo   = NULL;
    op.fFlags= FOF_NOCONFIRMATION | FOF_SILENT | FOF_NOERRORUI ;

    // Perform the deletion
    SHFileOperation(&op);

    // Set dir back to startup dir
    SetCurrentDir(StartupDirectory);

    // Update statusbar
    StatusBar->Panels->Items[1]->GaugeAttrs->Style = gsPercent;
    StatusBar->Panels->Items[1]->PanelType = sptNormal;
    StatusBar->Panels->Items[1]->Enabled = true;
    StatusBar->Panels->Items[1]->Alignment = taLeftJustify;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Comment1Click(TObject *Sender)
{
    // Init the Comment form
    CommentForm->Memo->Lines->BeginUpdate();
    CommentForm->Memo->Text = ZipBuilder->ZipComment;
    CommentForm->Memo->Lines->EndUpdate();
    CommentForm->Comment->Clear();

    // Show
    CommentForm->ShowModal();

    // Save comment?
    if (CommentForm->bSave)
    {
        ZipBuilder->ZipComment = CommentForm->Comment->Text;
    }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::Test1Click(TObject *Sender)
{
    // Set options
    ZipBuilder->ExtrOptions.Clear();
    ZipBuilder->ExtrOptions << ExtrTest;

    // Set FSpecArgs
    ZipBuilder->FSpecArgs->Clear();

    // Do the testing
    bTesting = true;
    iTestErrors = 0;
    ZipBuilder->Extract();
    bTesting = false;

    // Update the system
    Update();

    // Show test result
    // Improvement: Perform a scroolup to the top. Look at: EM_LINEINDEX
    TestForm->ShowModal();
}
//---------------------------------------------------------------------------
// Zip error message
void __fastcall TMainForm::ZipBuilderMessage(TObject *Sender, int ErrCode,
      AnsiString Message)
{
    if (bTesting)
    {
        if (ErrCode == 0)
            TestForm->Memo->Lines->Add(Message+"      OK");
        else
        {
            TestForm->Memo->Lines->Add(Message+"      Failed");
            iTestErrors++;
        }
    }
    else if (ErrCode == 590089)  // Operation cancelled
    {
        Application->MessageBox("Operation was cancelled!", "JZip", MB_OK | MB_ICONWARNING);
        Update();
        ExtListViewChange(NULL, NULL, NULL);

        // Set everything back to normal cursors
        Cursor = crDefault;
        ToolBar->Cursor = crDefault;
        ExtListView->Cursor = crDefault;
        StatusBar->Cursor = crDefault;
        File1->Enabled = true;
        Actions1->Enabled = true;
        Options1->Enabled = true;
        Help1->Enabled = true;
        ToolBar->Enabled = true;
        NewToolButton->Enabled = true;
        OpenToolButton->Enabled = true;
        AddToolButton->Enabled = true;
        ExtractToolButton->Enabled = true;
        AboutToolButton->Enabled = true;
        ExtListView->Enabled = true;
    }
    else if (ErrCode != 0)
    {
        Application->MessageBox(Message.c_str(), AnsiString("JZip - Error: "+IntToStr(ErrCode)).c_str(), MB_OK | MB_ICONERROR);
    }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::About1Click(TObject *Sender)
{
    // Show about box
    AboutBox->ShowModal();
}
//---------------------------------------------------------------------------
// Delete files in archive
void __fastcall TMainForm::Delete1Click(TObject *Sender)
{
    if (Application->MessageBox("Are you sure you want to delete the file(s)?", "Delete File(s)", MB_ICONQUESTION | MB_YESNO) == ID_NO)
        return;

    ZipBuilder->FSpecArgs->Clear();

    TListItem *SelItem = ExtListView->Selected;
    TItemStates SearchState;
    SearchState << isSelected;

    // Find all selected items
    while( SelItem )
    {
        if (SelItem->SubItems->Strings[4] == "")
            ZipBuilder->FSpecArgs->Add(SelItem->Caption);
        else
            ZipBuilder->FSpecArgs->Add(SelItem->SubItems->Strings[4]+"\\"+SelItem->Caption);

        SelItem = ExtListView->GetNextItem(SelItem, sdAll, SearchState);
    }

    // Delete them
    ZipBuilder->Delete();

    // Update
    Update();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::MakeEXEFile1Click(TObject *Sender)
{
    // Show Exe form
    EXEForm->ShowModal();
    if (EXEForm->ModalResult == mrCancel)
        return;

    ZipBuilder->SFXDefaultDir = EXEForm->NameEdit->Text;

    // perform convertion
    int rc = ZipBuilder->ConvertSFX();
    switch(rc)
    {
        case  0: //   On succes.
            Application->MessageBox("Self-Extracting Archive was created successfully!", "Done", MB_OK | MB_ICONINFORMATION);
            break;
        case -1: //   Error in creation of destination.
            Application->MessageBox("Failed!\nReason: Error in creation of destination.", "Self-Extracting Archive", MB_OK | MB_ICONERROR);
            break;
        case -2: //  Read or write error during copy. (Could be: Not enough space on disk)
            Application->MessageBox("Failed!\nReason: Read or write error during copy. (Could be: Not enough space on disk)", "Self-Extracting Archive", MB_OK | MB_ICONERROR);
            break;
        case -3: //  Error in open of source.
            Application->MessageBox("Failed!\nReason: Error in open of source.", "Self-Extracting Archive", MB_OK | MB_ICONERROR);
            break;
        case -9: //	General failure during copy.
            Application->MessageBox("Failed!\nReason: General failure during copy.", "Self-Extracting Archive", MB_OK | MB_ICONERROR);
            break;
    } // end switch
}
//---------------------------------------------------------------------------
// Create a shortcut in a given destination (O: Desktop, 1: Startmenu)
void __fastcall TMainForm::CreateShortCut(const AnsiString &file, int iDestination)
{
    IShellLink* pLink;
    IPersistFile* pPersistFile;
    LPMALLOC      ShellMalloc;
    LPITEMIDLIST  DesktopPidl;
    char DesktopDir[MAX_PATH];

    // We are going to create a pidl, and it will need to be
    // freed by the shell mallocator. Get the shell mallocator
    // object using API SHGetMalloc function. Return if failure.
    if(FAILED(SHGetMalloc(&ShellMalloc)))
        return;

    // Check for the destination of our shortcut
    switch(iDestination)
    {
        case 0: // Desktop
        // use the API to get a pidl for the desktop directory
        // if function fails, return without proceeding
        if(FAILED(SHGetSpecialFolderLocation(NULL,
                                             CSIDL_DESKTOPDIRECTORY,
                                             &DesktopPidl)))
            return;
        break;
        case 1: // Startmenu
        if(FAILED(SHGetSpecialFolderLocation(NULL,
                                             CSIDL_STARTMENU,
                                             &DesktopPidl)))
        return;
    }

    // Now convert the pidl to a character string
    // return if function fails
    if(!SHGetPathFromIDList(DesktopPidl, DesktopDir))
    {
        ShellMalloc->Free(DesktopPidl);
        ShellMalloc->Release();
        return;
    }

    // At this point, we are done with the pidl and the
    // mallocator, so free them up
    ShellMalloc->Free(DesktopPidl);
    ShellMalloc->Release();

    if(SUCCEEDED(CoInitialize(NULL)))
    {
        if(SUCCEEDED(CoCreateInstance(CLSID_ShellLink, NULL,
                                      CLSCTX_INPROC_SERVER,
                                      IID_IShellLink, (void **) &pLink)))
        {
            pLink->SetPath(file.c_str());
            pLink->SetDescription("JZip Archive Manager");
            pLink->SetShowCmd(SW_SHOW);

            if(SUCCEEDED(pLink->QueryInterface(IID_IPersistFile,
                                               (void **)&pPersistFile)))
            {

                WideString strShortCutLocation(DesktopDir);
                strShortCutLocation += "\\JZip.lnk";
                pPersistFile->Save(strShortCutLocation.c_bstr(), TRUE);
                pPersistFile->Release();
            }
            pLink->Release();
        }
        CoUninitialize();
    }
}
//----------------------------------------------------------------------
void __fastcall TMainForm::LicenseAgreement1Click(TObject *Sender)
{
    Application->MessageBox( "This software is provided \"as is\" without warranty of any kind, "
                                    "either expressed or implied.  the entire risk as to the "
                                    "quality and performance of the software is with you. Should the "
                                    "software prove defective, you assume the cost of all necessary "
                                    "servicing, repair, or correction.  in no event shall the author, "
                                    "copyright holder, or any other party who may redistribute the "
                                    "software be liable to you for damages, including any general, "
                                    "special, incidental, or consequential damages arising out of "
                                    "the use or inability to use the software (including, but not "
                                    "limited to, loss of data, data being rendered inaccurate, loss of "
                                    "business profits, loss of business information, business "
                                    "interruptions, loss sustained by you or third parties, or a "
                                    "failure of the software to operate with any other software) even "
                                    "if the author, copyright holder, or other party has been advised "
                                    "of the possibility of such damages.", "JZip Disclaimer",
                                    MB_OK | MB_ICONINFORMATION);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::OpenFile1Click(TObject *Sender)
{
    // Perform View
    View1Click(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Add2Click(TObject *Sender)
{
    // Perform Add
    Add1Click(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Delete2Click(TObject *Sender)
{
    // Perform Delete
    Delete1Click(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Extract2Click(TObject *Sender)
{
    // Perform Extract
    Extract1Click(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::View2Click(TObject *Sender)
{
    // Perform View
    View1Click(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SelectAll2Click(TObject *Sender)
{
    // Select All
    SelectAll1Click(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::InvertSelection2Click(TObject *Sender)
{
    // Invert Selection
    InvertSelection1Click(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::MakeEXEFile2Click(TObject *Sender)
{
    // Convert Zip to Exe file
    MakeEXEFile1Click(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Test2Click(TObject *Sender)
{
    // Perform Test
    Test1Click(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Comment2Click(TObject *Sender)
{
    // Show comment
    Comment1Click(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Install2Click(TObject *Sender)
{
    // Perform install
    Install1Click(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AboutToolButtonClick(TObject *Sender)
{
    // Show about box
    About1Click(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Exit1Click(TObject *Sender)
{
    // Close app
    Application->Terminate();
}

void __fastcall TMainForm::ExtListViewMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
    TPoint point;
    point.x = X;
    point.y = Y;

    // NB: There's problems using DragDetect.
    // After having dragged something from JZip to something else
    // the next click in the ListView will perform an extraction.
    // The authors of DragDropSuite hasn't been able to location the bug
    // in the source, so it's possible that the problem lies with DragDetect.
    // This should be investigated some more.
    // Notice: After build 192 (where the ExtListView was updated) and under the
    // new Windows 2000 Professional version (build 2195) the problem seems to
    // have vanished. Tests needs to be done under Win9x and Win NT 4.
    if (DragDetect(ExtListView->Handle, point))
    {
        DropFileTarget->Unregister();

        //clear any filenames left from a previous drag operation...
        DropFileSource->Files->Clear();
        ZipBuilder->FSpecArgs->Clear();

        /* --- Add the file names --- */
        // Create temp dir
        AnsiString TempDir = CreateTempDir();

        // Add the temp dir to the CleanUpList
        CleanUpList->Add(TempDir);
        ZipBuilder->ExtrBaseDir = TempDir;

        // Add files
        TListItem *SelItem = ExtListView->Selected;
        TItemStates SearchState;
        SearchState << isSelected;

        // Find selected item
        AnsiString asTmp;
        while( SelItem )
        {

            if (SelItem->SubItems->Strings[4] == "")
                asTmp = SelItem->Caption;
            else
                asTmp = SelItem->SubItems->Strings[4]+"\\"+SelItem->Caption;

            // Add it
            ZipBuilder->FSpecArgs->Add(asTmp);
            DropFileSource->Files->Add(TempDir+"\\"+asTmp);

            SelItem = ExtListView->GetNextItem(SelItem, sdAll, SearchState);
        }

        bDraggingOverUs = true;
        bDragging = true;

        //Do the dragdrop...
        DropFileSource->Execute();

        DropFileTarget->Register(ExtListView);
    }
    else
    {
        bDraggingOverUs = false;
        bDragging = false;
    }
}
//---------------------------------------------------------------------------


void __fastcall TMainForm::DropFileSourceDrop(TObject *Sender,
      TDragType DragType, bool &ContinueDrop)
{

    ContinueDrop = true;

    // Set options
    ZipBuilder->ExtrOptions.Clear();
    ZipBuilder->ExtrOptions << ExtrOverWrite;
    ZipBuilder->ExtrOptions << ExtrDirNames;

    // Extract
    ZipBuilder->Extract();
    Update();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Password1Click(TObject *Sender)
{
    // Password
    PasswordForm->Password1Edit->Text = "";
    PasswordForm->Password2Edit->Text = "";
    PasswordForm->ShowModal();

    if (PasswordForm->ModalResult == mrOk)
        ZipBuilder->Password = PasswordForm->Password1Edit->Text;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ZipBuilderPasswordError(TObject *Sender,
      AnsiString &NewPassword, AnsiString ForFile, unsigned &RepeatCount)
{
    // Password required
    ExtractPasswForm->PasswEdit->Text = "";
    ExtractPasswForm->ShowModal();
    if (ExtractPasswForm->ModalResult == mrOk)
    {
        NewPassword = ExtractPasswForm->PasswEdit->Text;
    }
    else
        NewPassword = "";

}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormResize(TObject *Sender)
{
    // Check if we should minimize the file name
    if (ZipBuilder->ZipFileName != "")
        Caption = "JZip - "+MinimizeName( ZipBuilder->ZipFileName, Canvas, ClientWidth-Canvas->TextWidth("JZip - ")-140);
        // NB: This should be improved some more. Was just a quick hack!
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Configuration1Click(TObject *Sender)
{
    /* --- Set the correct settings in the dialog --- */
    // View
    ConfigForm->FullRowCheckBox->Checked = bFullRowSelect;
    ConfigForm->GridLinesCheckBox->Checked = bGridLines;
    ConfigForm->FlatScroolbarsCheckBox->Checked = bFlatScroolBars;

    // Kept for later use (not implemented yet)
//    ConfigForm->UserCurrentRowCheckBox->Checked = bUseCurrentFolderSettings;

    if (bSingleClick)
    {
        ConfigForm->SingleRadioButton->Enabled = true;
        ConfigForm->DoubleRadioButton->Enabled = true;
        ConfigForm->AlwaysRadioButton->Enabled = true;
        ConfigForm->UnderlineRadioButton->Enabled = true;
        ConfigForm->NeverRadioButton->Enabled = true;

        ConfigForm->SingleRadioButton->Checked = true;

    }
    else // bDoubleClick
    {
        ConfigForm->SingleRadioButton->Enabled = true;
        ConfigForm->DoubleRadioButton->Enabled = true;
        ConfigForm->AlwaysRadioButton->Enabled = false;
        ConfigForm->UnderlineRadioButton->Enabled = false;
        ConfigForm->NeverRadioButton->Enabled = false;

        ConfigForm->DoubleRadioButton->Checked = true;
    }

    // Set SingleClick sub radiobuttons under all circumstances
    switch(iSingleClickOption)
    {
        case 0: // Never underline file names
            ConfigForm->NeverRadioButton->Checked = true;
            break;
        case 1: // Underline file names when I point at them
            ConfigForm->UnderlineRadioButton->Checked = true;
            break;
        case 2: // Always underline file names
            ConfigForm->AlwaysRadioButton->Checked = true;
            break;
    }


    // Toolbar
    ConfigForm->ShowButtonTextCheckBox->Checked = bShowButtonText;
    ConfigForm->ShowToolTipCheckBox->Checked = bShowToolTip;
    ConfigForm->FlatButtonsCheckBox->Checked = bFlatButtons;

    // System
    ConfigForm->AssociateCheckBox->Checked = bAssociateWithZIP;
    ConfigForm->IncludeInStartMenuCheckBox->Checked = bIncludeInStartMenu;
    ConfigForm->IncludeOnDesktopCheckBox->Checked = bIncludeOnDesktop;

    // -----------------------------------------------------------------------
    ConfigForm->ShowModal();
    if (ConfigForm->ModalResult == mrCancel)
        return;
    // -----------------------------------------------------------------------

    // Save settings ;)
    // View
    bFullRowSelect = ConfigForm->FullRowCheckBox->Checked;
    bGridLines = ConfigForm->GridLinesCheckBox->Checked;
    bFlatScroolBars = ConfigForm->FlatScroolbarsCheckBox->Checked;

    // Kept for later use (not implemented yet)
//   bUseCurrentFolderSettings = ConfigForm->UserCurrentRowCheckBox->Checked;

    bSingleClick = ConfigForm->SingleRadioButton->Checked;
    if (bSingleClick)
    {

        if (ConfigForm->NeverRadioButton->Checked)
            iSingleClickOption = 0;
        else
        if (ConfigForm->UnderlineRadioButton->Checked)
            iSingleClickOption = 1;
        else
        if (ConfigForm->AlwaysRadioButton->Checked)
            iSingleClickOption = 2;

        bDoubleClick = false;
    }
    else // bDoubleClick
    {
        bDoubleClick = true;
    }

    // Toolbar
    bShowButtonText = ConfigForm->ShowButtonTextCheckBox->Checked;
    bShowToolTip = ConfigForm->ShowToolTipCheckBox->Checked;
    bFlatButtons = ConfigForm->FlatButtonsCheckBox->Checked;

    // System
    bAssociateWithZIP = ConfigForm->AssociateCheckBox->Checked;
    bIncludeInStartMenu = ConfigForm->IncludeInStartMenuCheckBox->Checked;
    bIncludeOnDesktop = ConfigForm->IncludeOnDesktopCheckBox->Checked;

    // Make the changes effective
    SetConfiguration();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ReadConfiguration()
{
    TRegistry* thisReg = new TRegistry();

    thisReg->OpenKey("SOFTWARE\\JZip", true);

    // Read View settings
    if (thisReg->ValueExists("Full Row Select"))
        bFullRowSelect = thisReg->ReadBool("Full Row Select");
    else
        bFullRowSelect = true;  // Default value

    if (thisReg->ValueExists("Grid Lines"))
        bGridLines = thisReg->ReadBool("Grid Lines");
    else
        bGridLines = false;  // Default value

    if (thisReg->ValueExists("Flat Scroolbars"))
        bFlatScroolBars = thisReg->ReadBool("Flat Scrollbars");
    else
        bFlatScroolBars = false;  // Default value


    if (thisReg->ValueExists("Use Current Folder Settings"))
        bUseCurrentFolderSettings = thisReg->ReadBool("Use Current Folder Settings");
    else
        bUseCurrentFolderSettings = false;  // Default value

    if (thisReg->ValueExists("Use Current Folder Settings"))
        bUseCurrentFolderSettings = thisReg->ReadBool("Use Current Folder Settings");
    else
        bUseCurrentFolderSettings = false;  // Default value

    if (thisReg->ValueExists("Single Click"))
        bSingleClick = thisReg->ReadBool("Single Click");
    else
        bSingleClick = false;  // Default value

    if (thisReg->ValueExists("Single Click Option"))
        iSingleClickOption = thisReg->ReadInteger("Single Click Option");
    else
        iSingleClickOption = 1;  // Default value

    if (thisReg->ValueExists("Double Click"))
        bDoubleClick = thisReg->ReadBool("Double Click");
    else
        bDoubleClick = true;  // Default value


    // Toolbar
    if (thisReg->ValueExists("Show Button Text"))
        bShowButtonText = thisReg->ReadBool("Show Button Text");
    else
        bShowButtonText = true;  // Default value

    if (thisReg->ValueExists("Show ToolTip"))
        bShowToolTip = thisReg->ReadBool("Show ToolTip");
    else
        bShowToolTip = true;  // Default value

    if (thisReg->ValueExists("Flat Toolbar Buttons"))
        bFlatButtons = thisReg->ReadBool("Flat Toolbar Buttons");
    else
        bFlatButtons = true;  // Default value


    // System
    if (thisReg->ValueExists("Associate JZip with ZIP archives"))
        bAssociateWithZIP = thisReg->ReadBool("Associate JZip with ZIP archives");
    else
        bAssociateWithZIP = true;  // Default value

    if (thisReg->ValueExists("Include JZip in the Start Menu"))
        bIncludeInStartMenu = thisReg->ReadBool("Include JZip in the Start Menu");
    else
        bIncludeInStartMenu = true;  // Default value

    if (thisReg->ValueExists("Include JZip on the Desktop"))
        bIncludeOnDesktop = thisReg->ReadBool("Include JZip on the Desktop");
    else
        bIncludeOnDesktop = true;  // Default value

    // Clean up
    thisReg->CloseKey();
    delete thisReg;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SetConfiguration()
{
    // View
    if (bFullRowSelect)
        ExtListView->ExtendedStyles = ExtListView->ExtendedStyles << lvxFullRowSelect;
    else
        ExtListView->ExtendedStyles = ExtListView->ExtendedStyles  >> lvxFullRowSelect;

    if (bGridLines)
        ExtListView->ExtendedStyles = ExtListView->ExtendedStyles << lvxGridLines;
    else
        ExtListView->ExtendedStyles = ExtListView->ExtendedStyles >> lvxGridLines;

    if (bFlatScroolBars)
        ExtListView->ExtendedStyles = ExtListView->ExtendedStyles << lvxFlatScrollBar;
    else
        ExtListView->ExtendedStyles = ExtListView->ExtendedStyles >> lvxFlatScrollBar;

    //bUseCurrentFolderSettings (not implemented);

    if (bSingleClick)
    {
        ExtListView->ExtendedStyles = ExtListView->ExtendedStyles << lvxOneClickActivate;
        ExtListView->ExtendedStyles = ExtListView->ExtendedStyles >> lvxTwoClickActivate;
        ExtListView->ExtendedStyles = ExtListView->ExtendedStyles << lvxTrackSelect;
        switch (iSingleClickOption)
        {
            case 0:     // Never underline file names
                ExtListView->ExtendedStyles = ExtListView->ExtendedStyles >> lvxUnderlineHot;
                ExtListView->ExtendedStyles = ExtListView->ExtendedStyles >> lvxUnderlineCold;
                break;
            case 1:     // Underline file names when I point at them
                ExtListView->ExtendedStyles = ExtListView->ExtendedStyles << lvxUnderlineHot;
                ExtListView->ExtendedStyles = ExtListView->ExtendedStyles >> lvxUnderlineCold;
                break;
            case 2:     // Always underline file names
                ExtListView->ExtendedStyles = ExtListView->ExtendedStyles << lvxUnderlineCold;
                break;
        } // end switch
    }
    else // bDoubleClick
    {
        ExtListView->ExtendedStyles = ExtListView->ExtendedStyles >> lvxTrackSelect;
        ExtListView->ExtendedStyles = ExtListView->ExtendedStyles >> lvxUnderlineHot;
        ExtListView->ExtendedStyles = ExtListView->ExtendedStyles >> lvxUnderlineCold;
        ExtListView->ExtendedStyles = ExtListView->ExtendedStyles >> lvxOneClickActivate;
        ExtListView->ExtendedStyles = ExtListView->ExtendedStyles >> lvxTwoClickActivate;
    }

    // Toolbar
    ToolBar->ShowCaptions = bShowButtonText;
    Application->ShowHint = bShowToolTip;
    ToolBar->Flat = bFlatButtons;

    // System
    if (bAssociateWithZIP)
    {
        /* --- Check who owns the ZIP file --- */
        TRegistry* thisReg = new TRegistry();

        // Register zip file format
        thisReg->RootKey = HKEY_CLASSES_ROOT;
        thisReg->OpenKey(".zip", true);
        if (thisReg->ReadString("") == "JZip")
        {
            // We already own the extension
            thisReg->CloseKey();
            delete thisReg;
            goto CONTINUE;
        }
        thisReg->WriteString("", "JZip");
        thisReg->CloseKey();

        // Check in the Open command
        thisReg->OpenKey("JZip", true);
        thisReg->WriteString("", "JZip Archive");
        thisReg->OpenKey("shell", true);
        thisReg->WriteString("", "open");

        thisReg->OpenKey("open", true);
        thisReg->WriteString("", "&Open");
        thisReg->OpenKey("command", true);
        thisReg->WriteString("", ParamStr(0)+" \"\%1\"");
        thisReg->CloseKey();

        // Set default icon
        thisReg->OpenKey("JZip", true);
        thisReg->OpenKey("DefaultIcon", true);
        thisReg->WriteString("", ParamStr(0)+",0");

        thisReg->CloseKey();
        delete thisReg;
    }

CONTINUE:
    if (bIncludeInStartMenu)
    {
        CreateShortCut(ParamStr(0), iStartMenu);
    }
    else
    {
        // Delete the shortcut (if it exists)
        DeleteShortCut(iStartMenu);
    }

    if (bIncludeOnDesktop)
    {
        CreateShortCut(ParamStr(0), iDesktop);
    }
    else
    {
        // Delete the shortcut (if it exists)
        DeleteShortCut(iDesktop);
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WriteConfiguration()
{
    TRegistry* thisReg = new TRegistry();

    thisReg->OpenKey("SOFTWARE\\JZip", true);

    // Write View settings
    thisReg->WriteBool("Full Row Select", bFullRowSelect);
    thisReg->WriteBool("Grid Lines", bGridLines);
    thisReg->WriteBool("Flat Scrollbars", bFlatScroolBars);
    thisReg->WriteBool("Use Current Folder Settings", bUseCurrentFolderSettings);
    thisReg->WriteBool("Single Click", bSingleClick);
    thisReg->WriteInteger("Single Click Option", iSingleClickOption);
    thisReg->WriteBool("Double Click", bDoubleClick);

    // Toolbar
    thisReg->WriteBool("Show Button Text", bShowButtonText);
    thisReg->WriteBool("Show ToolTip", bShowToolTip);
    thisReg->WriteBool("Flat Toolbar Buttons", bFlatButtons);

    // System
    thisReg->WriteBool("Associate JZip with ZIP archives", bAssociateWithZIP);
    thisReg->WriteBool("Include JZip in the Start Menu", bIncludeInStartMenu);
    thisReg->WriteBool("Include JZip on the Desktop", bIncludeOnDesktop );

    // Clean up
    thisReg->CloseKey();
    delete thisReg;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DeleteShortCut(int iDestination)
{
    LPMALLOC      ShellMalloc;
    LPITEMIDLIST  DesktopPidl;
    char DesktopDir[MAX_PATH];

    // We are going to create a pidl, and it will need to be
    // freed by the shell mallocator. Get the shell mallocator
    // object using API SHGetMalloc function. Return if failure.
    if(FAILED(SHGetMalloc(&ShellMalloc)))
        return;

    // Check for the destination of our shortcut
    switch(iDestination)
    {
        case 0: // Desktop
        // use the API to get a pidl for the desktop directory
        // if function fails, return without proceeding
        if(FAILED(SHGetSpecialFolderLocation(NULL,
                                             CSIDL_DESKTOPDIRECTORY,
                                             &DesktopPidl)))
            return;
        break;
        case 1: // Startmenu
        if(FAILED(SHGetSpecialFolderLocation(NULL,
                                             CSIDL_STARTMENU,
                                             &DesktopPidl)))
        return;
    }

    // Now convert the pidl to a character string
    // return if function fails
    if(!SHGetPathFromIDList(DesktopPidl, DesktopDir))
    {
        ShellMalloc->Free(DesktopPidl);
        ShellMalloc->Release();
        return;
    }

    // At this point, we are done with the pidl and the
    // mallocator, so free them up
    ShellMalloc->Free(DesktopPidl);
    ShellMalloc->Release();

    DeleteFile(AnsiString(DesktopDir)+"\\JZip.lnk");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormShow(TObject *Sender)
{
    // Make sure the code below is only executed once. Therefore bFirstTime is
    // used.
    static bool bFirstTime = true;

    if (!bFirstTime)
        return;

    bFirstTime = false;

    /* --- Check if JZip was started with an argument --- */
    if (ParamCount() == 0)
        return;

    if (ParamCount() == 1 && ExtractFileExt(ParamStr(1)).LowerCase() == ".zip")
    {
        Caption = "JZip - "+MinimizeName( ParamStr(1), Canvas, ClientWidth-Canvas->TextWidth("JZip - ")-140);
        ZipBuilder->ZipFileName = ParamStr(1);
        Update();
        return;
    }


    bool bAddSpecial = false;   // Add subfolder but don't add extra folder info

    // Here we set the selections of the DragDropForm to those of the
    // Add form.
    // NB: The AddForm is responsible for loading Add settings, so therefore
    // it's import that the AddForm is created before the DragDropForm is used!
    DragDropForm->ActionComboBox->ItemIndex = AddForm->ActionComboBox->ItemIndex;
    DragDropForm->CompressionComboBox->ItemIndex = AddForm->CompressionComboBox->ItemIndex;
    DragDropForm->SubFldrCheckBox->Checked = AddForm->SubFldrCheckBox->Checked;
    DragDropForm->ExtraFldrCheckBox->Checked = AddForm->ExtraFldrCheckBox->Checked;
    DragDropForm->SysFilesCheckBox->Checked = AddForm->SysFilesCheckBox->Checked;
    DragDropForm->ForceDOSCheckBox->Checked = AddForm->ForceDOSCheckBox->Checked;

    DragDropForm->Password = "";
    // Set file label text
    if (DropFileTarget->Files->Count == 1)
    {
        DragDropForm->FileLabel->Caption = "Adding File: "+MinimizeName( DropFileTarget->Files->Strings[0], DragDropForm->FileLabel->Canvas, DragDropForm->FileLabel->Width-140);
    }
    else
    {
        DragDropForm->FileLabel->Caption = "Adding "+IntToStr(ParamCount())+" Files";
    }

    // Set initial dir and NameEdit
    AnsiString asTmp = ExtractFilePath(ParamStr(1));
    DragDropForm->NewDialog->InitialDir = asTmp;
    DragDropForm->OpenDialog->InitialDir = asTmp;
    DragDropForm->NameEdit->Text = asTmp;
    DragDropForm->AddBitBtn->Enabled = false;

    // Set title to Add (since we'll we reusing this
    // dialog for DragDrop too)
    DragDropForm->Caption = "Add";

    // Show DragDrop form
    DragDropForm->ShowModal();
    if (DragDropForm->ModalResult == mrCancel)
        return;

    // Since we're gonna use the settings from the Add form below (because
    // I'm lazy) and because we need to update the AddForm settings too,
    // we now set the selection in the AddForm to the selection from the
    // DragDrop form...
    AddForm->ActionComboBox->ItemIndex = DragDropForm->ActionComboBox->ItemIndex;
    AddForm->CompressionComboBox->ItemIndex = DragDropForm->CompressionComboBox->ItemIndex;
    AddForm->SubFldrCheckBox->Checked = DragDropForm->SubFldrCheckBox->Checked;
    AddForm->ExtraFldrCheckBox->Checked = DragDropForm->ExtraFldrCheckBox->Checked;
    AddForm->SysFilesCheckBox->Checked = DragDropForm->SysFilesCheckBox->Checked;
    AddForm->ForceDOSCheckBox->Checked = DragDropForm->ForceDOSCheckBox->Checked;
    AddForm->Password = DragDropForm->Password;

    // Set zip file name
    if (DragDropForm->NameEdit->Text[DragDropForm->NameEdit->Text.Length()] != '\\' && ExtractFileExt(DragDropForm->NameEdit->Text) == "")
    {
        ZipBuilder->ZipFileName = DragDropForm->NameEdit->Text+".zip";
    }
    else
        ZipBuilder->ZipFileName = DragDropForm->NameEdit->Text;

    Caption = "JZip - "+MinimizeName( ZipBuilder->ZipFileName, Canvas, ClientWidth-Canvas->TextWidth("JZip - ")-140);

    /* --- Steps that are default for both with and without wildcard --- */
    // Set compression level
    switch(AddForm->CompressionComboBox->ItemIndex)
    {
        case 0 : // Maximum
            ZipBuilder->AddCompLevel = 9;
            break;
        case 1 : // Normal
            ZipBuilder->AddCompLevel = 6;
            break;
        case 2 : // Fast
            ZipBuilder->AddCompLevel = 3;
            break;
        case 3 : // Superfast
            ZipBuilder->AddCompLevel = 2;
            break;
        case 4 : // None
            ZipBuilder->AddCompLevel = 0;
            break;
    } // end switch

    // Set Action type
    ZipBuilder->AddOptions.Clear();
    switch(AddForm->ActionComboBox->ItemIndex)
    {
        case 0: // Add (and replace)
            // Don nothing. This is the default
            break;
        case 1: // Freshen existing files
            ZipBuilder->AddOptions << AddFreshen;
            break;
        case 2: // Move files
            ZipBuilder->AddOptions << AddMove;
            break;
        case 3: // Update and add
            ZipBuilder->AddOptions << AddUpdate;
            break;
    } // end switch

    // Store 8.3 names
    if (AddForm->ForceDOSCheckBox->Checked)
        ZipBuilder->AddOptions << AddForceDOS;

    // Include SubFolders
    if (AddForm->SubFldrCheckBox->Checked)
    {
        ZipBuilder->AddOptions << AddRecurseDirs;
        ZipBuilder->AddOptions << AddDirNames;
    }

    // Save extra folder info
    if (AddForm->ExtraFldrCheckBox->Checked)
        ZipBuilder->AddOptions << AddDirNames;

    // Include System and hidden files
    if (AddForm->SysFilesCheckBox->Checked)
        ZipBuilder->AddOptions << AddHiddenFiles;

    // Password
    if (AddForm->Password != "")
        ZipBuilder->Password = AddForm->Password;
    if (ZipBuilder->Password != "")
        ZipBuilder->AddOptions << AddEncrypt;

    // Check for special request by user
    // Add subfolder but don't add extra folder info
    if (AddForm->SubFldrCheckBox->Checked && !AddForm->ExtraFldrCheckBox->Checked)
    {
        SetCurrentDir(ExtractFilePath(ParamStr(1)));
        bAddSpecial = true;
    }

    // Clear args
    ZipBuilder->FSpecArgs->Clear();

    for (int iLoop=0; iLoop < ParamCount(); iLoop++)
    {
        // Check if it's a directory we're adding
        if (AddForm->IsDirectory(ParamStr(iLoop+1)))
        {
            if (bAddSpecial)
                ZipBuilder->FSpecArgs->Add( ExtractFileName(ParamStr(iLoop+1))+"\\*.*");
            else
                ZipBuilder->FSpecArgs->Add( ParamStr(iLoop+1)+"\\*.*");
        }
        else
        {
            if (bAddSpecial)
                ZipBuilder->FSpecArgs->Add( ExtractFileName(ParamStr(iLoop+1)) );
            else
                ZipBuilder->FSpecArgs->Add( ParamStr(iLoop+1) );
        }
    }

    // Do the works ;)
    ZipBuilder->Add();

    // Update the system
    Update();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FormKeyPress(TObject *Sender, char &Key)
{
    if (Key == VK_ESCAPE && bInProgress)
    {
        if (Application->MessageBox("Press \"Yes\" if you wish to Abort the operation and \"No\" if you wish to Continue!\n\nNB: If you choose to Abort, the operation will first be aborted when JZip is done processing the current file..."
                                    , "Abort?", MB_YESNO | MB_ICONQUESTION)
            == IDYES)
        {
            ZipBuilder->Cancel = true;
            bInProgress = false;
        }
    }
}
//---------------------------------------------------------------------------
// Filename collision. Overwrite?
void __fastcall TMainForm::ZipBuilderExtractOverwrite(TObject *Sender,
      AnsiString ForFile, bool IsOlder, bool &DoOverwrite)
{
    static int handle;
    static int fileDate;

    if (ZipBuilder->Cancel)
    {
        DoOverwrite = false;
        return;
    }

    if (bYesToAll)
    {
        DoOverwrite = true;
        return;
    }

    // Orig file info
    OverwriteForm->OrigFileLabel->Caption = ZipBuilder->ExtrBaseDir+"\\"+ForFile;
    handle = open(OverwriteForm->OrigFileLabel->Caption.c_str(),O_RDONLY);
    OverwriteForm->OrigFileDataLabel->Caption = "";
    if (handle == -1)
    {
        OverwriteForm->OrigFileDataLabel->Caption = "";
    }
    else
    {
        long lFileSize = filelength(handle);
        close(handle);

        handle = FileOpen(ForFile, fmOpenRead);

        if (lFileSize == -1)
        {
            OverwriteForm->OrigFileDataLabel->Caption = "";
            goto END;
        }

        if (handle == -1)
        {
            OverwriteForm->OrigFileDataLabel->Caption = "";
            goto END;
        }

        if (lFileSize/1024 > 1)
        {
            OverwriteForm->OrigFileDataLabel->Caption = IntToStr((int)lFileSize/1024)+"KB ";
        }
        else
        {
            OverwriteForm->OrigFileDataLabel->Caption = IntToStr((int)lFileSize)+" bytes ";
        }

        fileDate = FileGetDate(handle);
        if (fileDate != -1)
            OverwriteForm->OrigFileDataLabel->Caption = OverwriteForm->OrigFileDataLabel->Caption + FormatDateTime( "ddddd  t", FileDateToDateTime( fileDate ) );
        FileClose(handle);

        END:
    }


    // New file info
    OverwriteForm->NewFileLabel->Caption = ExtractFileName(ForFile);
    OverwriteForm->NewFileDataLabel->Caption = "";
    ZipDirEntry *zde;
    for (int iLoop=0; iLoop < ZipBuilder->Count; iLoop++)
    {
        zde = (ZipDirEntry *)ZipBuilder->ZipContents->Items[ iLoop ];
        if (zde->FileName == ForFile)
        {
            if (zde->UncompressedSize/1024 > 1)
            {
                OverwriteForm->NewFileDataLabel->Caption = IntToStr((int)zde->UncompressedSize/1024)+"KB ";
            }
            else
            {
                OverwriteForm->NewFileDataLabel->Caption = IntToStr((int)zde->UncompressedSize)+" bytes ";
            }

            // Date
            try
            {
                OverwriteForm->NewFileDataLabel->Caption = OverwriteForm->NewFileDataLabel->Caption+FormatDateTime( "ddddd  t", FileDateToDateTime( zde->DateTime ) );
            }
            catch(...){}

            break;
        }
    }

    // Show the overwrite form
    OverwriteForm->ShowModal();

    // Perform according to user selection in OverwriteForm
    switch(OverwriteForm->ModalResult)
    {
        case mrYes :
            DoOverwrite = true;
            break;
        case mrAll :
            DoOverwrite = true;
            bYesToAll = true;
            break;
        case mrNo  :
            DoOverwrite = false;
            break;
        case mrCancel :
            DoOverwrite = false;
            ZipBuilder->Cancel = true;
            break;
    } // end switch
}
//---------------------------------------------------------------------------


void __fastcall TMainForm::DropFileTargetDrop(TObject *Sender,
      TShiftState ShiftState, TPoint &Point, int &Effect)
{
    bool bAddSpecial = false;   // Add subfolder but don't add extra folder info

    // Check if the dragged file is a zip file
    if (DropFileTarget->Files->Count == 1 && ExtractFileExt(DropFileTarget->Files->Strings[0]).LowerCase() == ".zip")
    {
        Caption = "JZip - "+MinimizeName( DropFileTarget->Files->Strings[0], Canvas, ClientWidth-Canvas->TextWidth("JZip - ")-140);
        ZipBuilder->ZipFileName = DropFileTarget->Files->Strings[0];
        Update();
        return;
    }

    // Check for a ZIP filename
    if (ZipBuilder->ZipFileName == "")
    {
        // Here we set the selections of the DragDropForm to those of the
        // Add form.
        // NB: The AddForm is responsible for loading Add settings, so therefore
        // it's import that the AddForm is created before the DragDropForm is used!
        DragDropForm->ActionComboBox->ItemIndex = AddForm->ActionComboBox->ItemIndex;
        DragDropForm->CompressionComboBox->ItemIndex = AddForm->CompressionComboBox->ItemIndex;
        DragDropForm->SubFldrCheckBox->Checked = AddForm->SubFldrCheckBox->Checked;
        DragDropForm->ExtraFldrCheckBox->Checked = AddForm->ExtraFldrCheckBox->Checked;
        DragDropForm->SysFilesCheckBox->Checked = AddForm->SysFilesCheckBox->Checked;
        DragDropForm->ForceDOSCheckBox->Checked = AddForm->ForceDOSCheckBox->Checked;

        DragDropForm->Password = "";
        // Set file label text
        if (DropFileTarget->Files->Count == 1)
        {
            DragDropForm->FileLabel->Caption = "File Dropped: "+MinimizeName( DropFileTarget->Files->Strings[0], DragDropForm->FileLabel->Canvas, DragDropForm->FileLabel->Width-140);
        }
        else
        {
            DragDropForm->FileLabel->Caption = "Files Dropped: "+IntToStr(DropFileTarget->Files->Count);
        }

        // Set initial dir and NameEdit
        AnsiString asTmp = ExtractFilePath(DropFileTarget->Files->Strings[0]);
        DragDropForm->NewDialog->InitialDir = asTmp;
        DragDropForm->OpenDialog->InitialDir = asTmp;
        DragDropForm->NameEdit->Text = asTmp;
        DragDropForm->AddBitBtn->Enabled = false;

        // Set title to Drag and Drop (since we'll we reusing this
        // dialog for Add (files dropped on the icon) too)
        DragDropForm->Caption = "Drag and Drop";

        DragDropForm->ShowModal();
        if (DragDropForm->ModalResult == mrCancel)
            return;

        // Since we're gonna use the settings from the Add form below (because
        // I'm lazy) and because we need to update the AddForm settings too,
        // we now set the selection in the AddForm to the selection from the
        // DragDrop form...
        AddForm->ActionComboBox->ItemIndex = DragDropForm->ActionComboBox->ItemIndex;
        AddForm->CompressionComboBox->ItemIndex = DragDropForm->CompressionComboBox->ItemIndex;
        AddForm->SubFldrCheckBox->Checked = DragDropForm->SubFldrCheckBox->Checked;
        AddForm->ExtraFldrCheckBox->Checked = DragDropForm->ExtraFldrCheckBox->Checked;
        AddForm->SysFilesCheckBox->Checked = DragDropForm->SysFilesCheckBox->Checked;
        AddForm->ForceDOSCheckBox->Checked = DragDropForm->ForceDOSCheckBox->Checked;
        AddForm->Password = DragDropForm->Password;

        // Set zip file name
        if (DragDropForm->NameEdit->Text[DragDropForm->NameEdit->Text.Length()] != '\\' && ExtractFileExt(DragDropForm->NameEdit->Text) == "")
        {
            ZipBuilder->ZipFileName = DragDropForm->NameEdit->Text+".zip";
        }
        else
            ZipBuilder->ZipFileName = DragDropForm->NameEdit->Text;

        Caption = "JZip - "+MinimizeName( ZipBuilder->ZipFileName, Canvas, ClientWidth-Canvas->TextWidth("JZip - ")-140);
    }

    /* --- Steps that are default for both with and without wildcard --- */
    // Set compression level
    switch(AddForm->CompressionComboBox->ItemIndex)
    {
        case 0 : // Maximum
            ZipBuilder->AddCompLevel = 9;
            break;
        case 1 : // Normal
            ZipBuilder->AddCompLevel = 6;
            break;
        case 2 : // Fast
            ZipBuilder->AddCompLevel = 3;
            break;
        case 3 : // Superfast
            ZipBuilder->AddCompLevel = 2;
            break;
        case 4 : // None
            ZipBuilder->AddCompLevel = 0;
            break;
    } // end switch

    // Set Action type
    ZipBuilder->AddOptions.Clear();
    switch(AddForm->ActionComboBox->ItemIndex)
    {
        case 0: // Add (and replace)
            // Don nothing. This is the default
            break;
        case 1: // Freshen existing files
            ZipBuilder->AddOptions << AddFreshen;
            break;
        case 2: // Move files
            ZipBuilder->AddOptions << AddMove;
            break;
        case 3: // Update and add
            ZipBuilder->AddOptions << AddUpdate;
            break;
    } // end switch

    // Store 8.3 names
    if (AddForm->ForceDOSCheckBox->Checked)
        ZipBuilder->AddOptions << AddForceDOS;

    // Include SubFolders
    if (AddForm->SubFldrCheckBox->Checked)
    {
        ZipBuilder->AddOptions << AddRecurseDirs;
        ZipBuilder->AddOptions << AddDirNames;
    }

    // Save extra folder info
    if (AddForm->ExtraFldrCheckBox->Checked)
        ZipBuilder->AddOptions << AddDirNames;

    // Include System and hidden files
    if (AddForm->SysFilesCheckBox->Checked)
        ZipBuilder->AddOptions << AddHiddenFiles;

    // Password
    if (AddForm->Password != "")
        ZipBuilder->Password = AddForm->Password;
    if (ZipBuilder->Password != "")
        ZipBuilder->AddOptions << AddEncrypt;

    // Check for special request by user
    // Add subfolder but don't add extra folder info
    if (AddForm->SubFldrCheckBox->Checked && !AddForm->ExtraFldrCheckBox->Checked)
    {
        SetCurrentDir(ExtractFilePath(DropFileTarget->Files->Strings[0]));
        bAddSpecial = true;
    }


    ZipBuilder->FSpecArgs->Clear();
    bool bIsDir;
    for (int iLoop=0; iLoop < DropFileTarget->Files->Count; iLoop++)
    {
        // Check if it's a directory we're adding
        bIsDir = AddForm->IsDirectory(DropFileTarget->Files->Strings[iLoop]);
        if (bIsDir)
        {
            if (bAddSpecial)
                ZipBuilder->FSpecArgs->Add( ExtractFileName(DropFileTarget->Files->Strings[iLoop])+"\\*.*");
            else
                ZipBuilder->FSpecArgs->Add( DropFileTarget->Files->Strings[iLoop]+"\\*.*");
        }
        else
        {
            if (bAddSpecial)
                ZipBuilder->FSpecArgs->Add( ExtractFileName(DropFileTarget->Files->Strings[iLoop]) );
            else
                ZipBuilder->FSpecArgs->Add( DropFileTarget->Files->Strings[iLoop] );
        }
    }

    // Set focus
    SetFocus();

    // Perform the add
    ZipBuilder->Add();

    // Update system
    Update();
    
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::TodoList1Click(TObject *Sender)
{
    ShellExecute(Application->Handle,
                "open",
                "http://www.bytamin-c.com",
                NULL,NULL,SW_SHOWDEFAULT);
}
//---------------------------------------------------------------------------

